import { __assign } from "tslib";
import { fabric } from 'fabric-all-modules';
import { getCursor } from './cursors';
export var modes = {
    PENCIL: 'PENCIL',
    LINE: 'LINE',
    RECTANGLE: 'RECTANGLE',
    TRIANGLE: 'TRIANGLE',
    ELLIPSE: 'ELLIPSE',
    ERASER: 'ERASER',
    SELECT: 'SELECT',
    TEXT: 'TEXT',
};
var Board = (function () {
    function Board(params) {
        var _this = this;
        this.cursorPencil = getCursor('pencil');
        this.mouseDown = false;
        this.drawInstance = null;
        this.canvasConfig = {
            zoom: 1,
            contentJSON: null,
            minZoom: 0.05,
            maxZoom: 9.99,
            viewportTransform: [1, 0, 0, 1, 0, 0],
        };
        if (params) {
            this.drawingSettings = params.drawingSettings;
        }
        this.canvas = this.initCanvas(this.canvasConfig);
        this.canvas.once('after:render', function () {
            _this.applyCanvasConfig(params.canvasConfig);
        });
        this.modes = modes;
        this.resetZoom();
        this.setDrawingMode(this.drawingSettings.currentMode);
        this.addZoomListeners();
    }
    Board.prototype.initCanvas = function () {
        fabric.Canvas.prototype.getItemByAttr = function (attr, name) {
            var object = null, objects = this.getObjects();
            for (var i = 0, len = this.size(); i < len; i++) {
                if (objects[i][attr] && objects[i][attr] === name) {
                    object = objects[i];
                    break;
                }
            }
            return object;
        };
        var canvasElement = document.getElementById('canvas');
        if (!canvasElement)
            return;
        var parentElement = canvasElement.parentNode;
        var canvas = new fabric.Canvas('canvas');
        canvas.perPixelTargetFind = true;
        if (parentElement) {
            this.element = this.handleResize(this.resizeCanvas(canvas, parentElement).bind(this));
            this.element.observe(parentElement);
        }
        return canvas;
    };
    Board.prototype.applyCanvasConfig = function (canvasConfig) {
        this.canvasConfig = __assign(__assign({}, this.canvasConfig), canvasConfig);
        if (this.canvasConfig.zoom) {
            this.canvas.setZoom(this.canvasConfig.zoom);
        }
        if (this.canvasConfig.contentJSON) {
            this.canvas.loadFromJSON(this.canvasConfig.contentJSON);
        }
        if (this.canvasConfig.viewportTransform) {
            this.canvas.viewportTransform = this.canvasConfig.viewportTransform;
            this.changeZoom({ scale: 1 });
        }
        this.canvas.requestRenderAll();
        console.log(this.canvas.getObjects());
        this.canvas.fire('config:chnage');
    };
    Board.prototype.addZoomListeners = function () {
        var canvas = this.canvas;
        var that = this;
        canvas.off('mouse:wheel');
        canvas.off('touch:gesture');
        canvas.on('mouse:wheel', function (opt) {
            opt.e.preventDefault();
            opt.e.stopPropagation();
            if (opt.e.ctrlKey) {
                var delta = opt.e.deltaY;
                var scale = Math.pow(0.995, delta);
                var point = { x: opt.e.offsetX, y: opt.e.offsetY };
                that.changeZoom({ point: point, scale: scale });
            }
            else {
                var e = opt.e;
                var vpt = canvas.viewportTransform;
                vpt[4] -= e.deltaX;
                vpt[5] -= e.deltaY;
                canvas.requestRenderAll();
            }
        });
        canvas.on('touch:gesture', function (event) {
            console.log('1 touch:gesture');
            if (event.e.touches && event.e.touches.length === 2) {
                var point1_1 = {
                    x: event.e.touches[0].clientX,
                    y: event.e.touches[0].clientY,
                };
                var point2_1 = {
                    x: event.e.touches[1].clientX,
                    y: event.e.touches[1].clientY,
                };
                var prevDistance_1 = canvas.getPointerDistance(point1_1, point2_1);
                canvas.on('touch:gesture', function (event) {
                    console.log('2 touch:gesture');
                    var newDistance = canvas.getPointerDistance(point1_1, point2_1);
                    var zoom = newDistance / prevDistance_1;
                    var point = {
                        x: (point1_1.x + point2_1.x) / 2,
                        y: (point1_1.y + point2_1.y) / 2,
                    };
                    var scale = zoom;
                    that.changeZoom({ point: point, scale: scale });
                    canvas.renderAll();
                    prevDistance_1 = newDistance;
                });
            }
        });
    };
    Board.prototype.setDrawingSettings = function (drawingSettings) {
        if (!drawingSettings)
            return;
        this.drawingSettings = __assign(__assign({}, this.drawingSettings), drawingSettings);
        this.setDrawingMode(this.drawingSettings.currentMode);
    };
    Board.prototype.setCanvasConfig = function (canvasConfig) {
        if (!canvasConfig)
            return;
        this.applyCanvasConfig(canvasConfig);
    };
    Board.prototype.setDrawingMode = function (mode) {
        this.drawingSettings.currentMode = mode;
        this.resetCanvas();
        switch (mode) {
            case this.modes.PENCIL:
                this.draw();
                break;
            case this.modes.LINE:
                this.createLine();
                break;
            case this.modes.RECTANGLE:
                this.createRect();
                break;
            case this.modes.ELLIPSE:
                this.createEllipse();
                break;
            case this.modes.TRIANGLE:
                this.createTriangle();
                break;
            case this.modes.ERASER:
                this.eraserOn();
                break;
            case this.modes.SELECT:
                this.onSelectMode();
                break;
            case this.modes.TEXT:
                this.createText();
                break;
            default:
                this.draw();
        }
    };
    Board.prototype.resetCanvas = function () {
        var canvas = this.canvas;
        this.removeCanvasListener(canvas);
        canvas.selection = false;
        canvas.isDrawingMode = false;
        canvas.defaultCursor = 'auto';
        canvas.hoverCursor = 'auto';
        canvas.getObjects().map(function (item) { return item.set({ selectable: false }); });
        if (this.editedTextObject) {
            this.editedTextObject.exitEditing();
            this.editedTextObject = null;
        }
    };
    Board.prototype.throttle = function (f, delay) {
        if (delay === void 0) { delay = 300; }
        var timer = 0;
        return function () {
            var _this = this;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            clearTimeout(timer);
            timer = setTimeout(function () { return f.apply(_this, args); }, delay);
        };
    };
    Board.prototype.handleResize = function (callback) {
        var resize_ob = new ResizeObserver(this.throttle(callback, 300));
        return resize_ob;
    };
    Board.prototype.resizeCanvas = function (canvas, whiteboard) {
        return function () {
            var width = whiteboard.clientWidth;
            var height = whiteboard.clientHeight;
            this.changeZoom({ scale: 1 });
            canvas.setDimensions({ width: width, height: height });
        };
    };
    Board.prototype.removeCanvasListener = function () {
        var canvas = this.canvas;
        canvas.off('mouse:down');
        canvas.off('mouse:move');
        canvas.off('mouse:up');
        canvas.off('mouse:over');
    };
    Board.prototype.draw = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        canvas.freeDrawingBrush = new fabric.PencilBrush(canvas);
        canvas.freeDrawingBrush.width = drawingSettings.brushWidth;
        canvas.freeDrawingBrush.color = drawingSettings.currentColor;
        canvas.isDrawingMode = true;
        canvas.freeDrawingCursor = this.cursorPencil;
    };
    Board.prototype.createLine = function () {
        var canvas = this.canvas;
        canvas.on('mouse:down', this.startAddLine().bind(this));
        canvas.on('mouse:move', this.startDrawingLine().bind(this));
        canvas.on('mouse:up', this.stopDrawing.bind(this));
        canvas.defaultCursor = this.cursorPencil;
        canvas.hoverCursor = this.cursorPencil;
        canvas.discardActiveObject().requestRenderAll();
    };
    Board.prototype.startAddLine = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        return function (_a) {
            var e = _a.e;
            this.mouseDown = true;
            var pointer = canvas.getPointer(e);
            this.drawInstance = new fabric.Line([pointer.x, pointer.y, pointer.x, pointer.y], {
                strokeWidth: drawingSettings.brushWidth,
                stroke: drawingSettings.currentColor,
                selectable: false,
            });
            canvas.add(this.drawInstance);
            canvas.requestRenderAll();
        };
    };
    Board.prototype.startDrawingLine = function () {
        var canvas = this.canvas;
        return function (_a) {
            var e = _a.e;
            if (this.mouseDown) {
                var pointer = canvas.getPointer(e);
                this.drawInstance.set({
                    x2: pointer.x,
                    y2: pointer.y,
                });
                this.drawInstance.setCoords();
                canvas.requestRenderAll();
            }
        };
    };
    Board.prototype.createRect = function () {
        var canvas = this.canvas;
        canvas.isDrawingMode = true;
        canvas.on('mouse:down', this.startAddRect().bind(this));
        canvas.on('mouse:move', this.startDrawingRect().bind(this));
        canvas.on('mouse:up', this.stopDrawing.bind(this));
        canvas.selection = false;
        canvas.defaultCursor = this.cursorPencil;
        canvas.hoverCursor = this.cursorPencil;
        canvas.isDrawingMode = false;
        canvas.getObjects().map(function (item) { return item.set({ selectable: false }); });
        canvas.discardActiveObject().requestRenderAll();
    };
    Board.prototype.startAddRect = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        return function (_a) {
            var e = _a.e;
            this.mouseDown = true;
            var pointer = canvas.getPointer(e);
            this.origX = pointer.x;
            this.origY = pointer.y;
            this.drawInstance = new fabric.Rect({
                stroke: drawingSettings.currentColor,
                strokeWidth: drawingSettings.brushWidth,
                fill: drawingSettings.fill ? drawingSettings.currentColor : 'transparent',
                left: this.origX,
                top: this.origY,
                width: 0,
                height: 0,
                selectable: false,
            });
            canvas.add(this.drawInstance);
            this.drawInstance.on('mousedown', function (e) {
                if (drawingSettings.currentMode === this.modes.ERASER) {
                    canvas.remove(e.target);
                }
            }.bind(this));
        };
    };
    Board.prototype.startDrawingRect = function () {
        var canvas = this.canvas;
        return function (_a) {
            var e = _a.e;
            if (this.mouseDown) {
                var pointer = canvas.getPointer(e);
                if (pointer.x < this.origX) {
                    this.drawInstance.set('left', pointer.x);
                }
                if (pointer.y < this.origY) {
                    this.drawInstance.set('top', pointer.y);
                }
                this.drawInstance.set({
                    width: Math.abs(pointer.x - this.origX),
                    height: Math.abs(pointer.y - this.origY),
                });
                this.drawInstance.setCoords();
                canvas.renderAll();
            }
        };
    };
    Board.prototype.stopDrawing = function () {
        this.mouseDown = false;
    };
    Board.prototype.createEllipse = function () {
        var canvas = this.canvas;
        canvas.isDrawingMode = true;
        canvas.on('mouse:down', this.startAddEllipse().bind(this));
        canvas.on('mouse:move', this.startDrawingEllipse().bind(this));
        canvas.on('mouse:up', this.stopDrawing.bind(this));
        canvas.selection = false;
        canvas.defaultCursor = this.cursorPencil;
        canvas.hoverCursor = this.cursorPencil;
        canvas.isDrawingMode = false;
        canvas.getObjects().map(function (item) { return item.set({ selectable: false }); });
        canvas.discardActiveObject().requestRenderAll();
    };
    Board.prototype.startAddEllipse = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        return function (_a) {
            var e = _a.e;
            this.mouseDown = true;
            var pointer = canvas.getPointer(e);
            this.origX = pointer.x;
            this.origY = pointer.y;
            this.drawInstance = new fabric.Ellipse({
                stroke: drawingSettings.currentColor,
                strokeWidth: drawingSettings.brushWidth,
                fill: drawingSettings.fill ? drawingSettings.currentColor : 'transparent',
                left: this.origX,
                top: this.origY,
                cornerSize: 7,
                objectCaching: false,
                selectable: false,
            });
            canvas.add(this.drawInstance);
        };
    };
    Board.prototype.startDrawingEllipse = function () {
        var canvas = this.canvas;
        return function (_a) {
            var e = _a.e;
            if (this.mouseDown) {
                var pointer = canvas.getPointer(e);
                if (pointer.x < this.origX) {
                    this.drawInstance.set('left', pointer.x);
                }
                if (pointer.y < this.origY) {
                    this.drawInstance.set('top', pointer.y);
                }
                this.drawInstance.set({
                    rx: Math.abs(pointer.x - this.origX) / 2,
                    ry: Math.abs(pointer.y - this.origY) / 2,
                });
                this.drawInstance.setCoords();
                canvas.renderAll();
            }
        };
    };
    Board.prototype.createTriangle = function () {
        var canvas = this.canvas;
        canvas.isDrawingMode = true;
        canvas.on('mouse:down', this.startAddTriangle().bind(this));
        canvas.on('mouse:move', this.startDrawingTriangle().bind(this));
        canvas.on('mouse:up', this.stopDrawing.bind(this));
        canvas.selection = false;
        canvas.defaultCursor = this.cursorPencil;
        canvas.hoverCursor = this.cursorPencil;
        canvas.isDrawingMode = false;
        canvas.getObjects().map(function (item) { return item.set({ selectable: false }); });
        canvas.discardActiveObject().requestRenderAll();
    };
    Board.prototype.startAddTriangle = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        return function (_a) {
            var e = _a.e;
            this.mouseDown = true;
            drawingSettings.currentMode = this.modes.TRIANGLE;
            var pointer = canvas.getPointer(e);
            this.origX = pointer.x;
            this.origY = pointer.y;
            this.drawInstance = new fabric.Triangle({
                stroke: drawingSettings.currentColor,
                strokeWidth: drawingSettings.brushWidth,
                fill: drawingSettings.fill ? drawingSettings.currentColor : 'transparent',
                left: this.origX,
                top: this.origY,
                width: 0,
                height: 0,
                selectable: false,
            });
            canvas.add(this.drawInstance);
        };
    };
    Board.prototype.startDrawingTriangle = function () {
        var canvas = this.canvas;
        return function (_a) {
            var e = _a.e;
            if (this.mouseDown) {
                var pointer = canvas.getPointer(e);
                if (pointer.x < this.origX) {
                    this.drawInstance.set('left', pointer.x);
                }
                if (pointer.y < this.origY) {
                    this.drawInstance.set('top', pointer.y);
                }
                this.drawInstance.set({
                    width: Math.abs(pointer.x - this.origX),
                    height: Math.abs(pointer.y - this.origY),
                });
                this.drawInstance.setCoords();
                canvas.renderAll();
            }
        };
    };
    Board.prototype.createText = function () {
        var _this = this;
        var canvas = this.canvas;
        canvas.isDrawingMode = true;
        canvas.on('mouse:down', function (e) { return _this.addText.call(_this, e); });
        canvas.isDrawingMode = false;
    };
    Board.prototype.addText = function (e) {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        var pointer = canvas.getPointer(e);
        this.origX = pointer.x;
        this.origY = pointer.y;
        var text = new fabric.Textbox('', {
            left: this.origX - 10,
            top: this.origY - 10,
            fontSize: drawingSettings.brushWidth * 3 + 10,
            fill: drawingSettings.currentColor,
            editable: true,
            perPixelTargetFind: false,
            keysMap: {
                13: 'exitEditing',
            },
        });
        canvas.add(text);
        canvas.renderAll();
        text.enterEditing();
        this.editedTextObject = text;
        canvas.off('mouse:down');
        canvas.once('mouse:down', function (e1) {
            var _this = this;
            if (text.isEditing) {
                text.exitEditing();
                this.editedTextObject = null;
                canvas.once('mouse:down', function (e2) {
                    _this.addText.call(_this, e2);
                });
            }
            else {
                this.addText.call(this, e1);
            }
        }.bind(this));
    };
    Board.prototype.eraserOn = function () {
        var canvas = this.canvas;
        canvas.isDrawingMode = false;
        canvas.on('mouse:down', function (event) {
            canvas.remove(event.target);
            canvas.on('mouse:move', function (e) {
                canvas.remove(e.target);
            });
        });
        canvas.on('mouse:up', function () {
            canvas.off('mouse:move');
        });
        canvas.on('mouse:over', function (event) {
            var hoveredObject = event.target;
            console.log(hoveredObject);
            if (hoveredObject) {
                hoveredObject.set({
                    opacity: 0.2,
                });
                canvas.requestRenderAll();
            }
        });
        canvas.on('mouse:out', function (event) {
            var hoveredObject = event.target;
            if (hoveredObject) {
                hoveredObject.set({
                    opacity: 1,
                });
                canvas.requestRenderAll();
            }
        });
        canvas.defaultCursor = getCursor('eraser');
        canvas.hoverCursor = getCursor('eraser');
    };
    Board.prototype.onSelectMode = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        drawingSettings.currentMode = '';
        canvas.isDrawingMode = false;
        canvas.getObjects().map(function (item) { return item.set({ selectable: true }); });
        canvas.hoverCursor = 'all-scroll';
    };
    Board.prototype.clearCanvas = function () {
        var canvas = this.canvas;
        canvas.getObjects().forEach(function (item) {
            if (item !== canvas.backgroundImage) {
                canvas.remove(item);
            }
        });
        canvas.setBackgroundImage(null, canvas.renderAll.bind(canvas));
    };
    Board.prototype.changeZoom = function (_a) {
        var point = _a.point, scale = _a.scale;
        if (!point) {
            var width = this.canvas.width;
            var height = this.canvas.height;
            point = { x: width / 2, y: height / 2 };
        }
        var minZoom = this.canvasConfig.minZoom;
        var maxZoom = this.canvasConfig.maxZoom;
        scale = this.canvas.getZoom() * scale;
        scale = scale < minZoom ? minZoom : scale > maxZoom ? maxZoom : scale;
        this.canvas.zoomToPoint(point, scale);
        this.onZoom({ point: point, scale: scale });
    };
    Board.prototype.resetZoom = function () {
        var width = this.canvas.width;
        var height = this.canvas.height;
        var point = { x: width / 2, y: height / 2 };
        var scale = 1;
        this.canvas.zoomToPoint(point, scale);
        this.onZoom({ point: point, scale: scale });
    };
    Board.prototype.onZoom = function (params) {
        this.addZoomListeners();
        this.canvas.fire('zoom:change', params);
    };
    Board.prototype.openPage = function (page) {
        var canvas = this.canvas;
        var center = canvas.getCenter();
        fabric.Image.fromURL(page, function (img) {
            if (img.width > img.height) {
                img.scaleToWidth(canvas.width);
            }
            else {
                img.scaleToHeight(canvas.height - 100);
            }
            canvas.setBackgroundImage(img, canvas.renderAll.bind(canvas), {
                top: center.top,
                left: center.left,
                originX: 'center',
                originY: 'center',
            });
            canvas.renderAll();
        });
    };
    Board.prototype.getCanvasContentBoundaries = function () {
        var canvas = this.canvas;
        var objects = canvas.getObjects();
        var minX = 10000;
        var minY = 10000;
        var maxX = -10000;
        var maxY = -10000;
        objects.forEach(function (object) {
            var objectBoundingBox = object.getBoundingRect();
            minX = Math.min(minX, objectBoundingBox.left);
            minY = Math.min(minY, objectBoundingBox.top);
            maxX = Math.max(maxX, objectBoundingBox.left + objectBoundingBox.width);
            maxY = Math.max(maxY, objectBoundingBox.top + objectBoundingBox.height);
        });
        var width = maxX - minX;
        var height = maxY - minY;
        return { minX: minX, minY: minY, maxX: maxX, maxY: maxY, width: width, height: height };
    };
    Board.prototype.removeBoard = function () {
        this.element.disconnect();
        if (this.canvas) {
            this.canvas.off();
            this.canvas.dispose();
        }
        this.canvas = null;
    };
    return Board;
}());
export { Board };
//# sourceMappingURL=Board.Class.js.map